/*
 * Simson's XML output class.
 */

#ifndef _XML_H_
#define _XML_H_

#include <string>
#include <stack>
#include <stdio.h>
#include <set>
#include <sys/types.h>

#ifndef WIN32
#include <pwd.h>
#endif

#ifdef HAVE_LIBAFFLIB
#include <afflib/afflib.h>
#endif

#ifdef HAVE_SYS_UTSNAME_H
#include <sys/utsname.h>
#endif

#ifdef HAVE_LIBAFFLIB
#include <afflib/afflib.h>
#endif

class xml {
    std::string outfilename;
    FILE  *out;				// where it is being written
    std::set<std::string> tags;			// XML tags
    void  write_doctype(FILE *out);
    void  write_dtd(FILE *out);
    bool  make_dtd;
    std::string  tempfilename;
    void  verify_tag(std::string tag);
    std::stack<std::string>tag_stack;
    void spaces();			// print spaces corresponding to tag stack
    std::string tempfile_template;

public:
    xml();				// private constructor

    void set_outFILE(FILE *out);	  // writes to this FILE without a DTD
    void set_outfilename(std::string outfilename);     // writes to this outfile with a DTD (needs a temp file)
    void set_makeDTD(bool flag);		 // should we write the DTD?
    void set_tempfile_template(std::string temp);

    static std::string xmlescape(const std::string &xml);
    static std::string xmlstrip(const std::string &xml);
    
    void open();			// opens the output file
    void close();			// writes the output to the file
    void tagout(const std::string &tag,const std::string &attribute);
    void xmlout(const std::string &tag,const std::string &value,
		const std::string &attribute,const bool escape_value);
    void xmlout(const std::string &tag,const std::string &value){
	xmlout(tag,value,"",true);
    }
    void xmlprintf(const std::string &tag,const std::string &attribute,
		   const char *fmt,...);
    void push(const std::string &tag,const std::string &attribute);
    void push(const std::string &tag) {push(tag,"");}
    void puts(const std::string &pdata);	// writes a std::string as parsed data
    void printf(const char *fmt,...);	// writes a std::string as parsed data
    void pop();	// close the tag
    void comment(const std::string &comment);

/* These support Digital Forensics XML and require certain variables to be defined */
    void add_DFXML_build_environment(){
	push("build_environment");
	xmlprintf("compiler","","GCC %d.%d",__GNUC__, __GNUC_MINOR__);
	xmlout("compilation_date",__DATE__ " " __TIME__);
#ifdef HAVE_LIBTSK3
	xmlout("library", "", std::string("name=\"tsk\" version=\"") + tsk_version_get_str() + "\"",false);
#endif
#ifdef HAVE_LIBAFFLIB
	xmlout("library", "", std::string("name=\"afflib\" version=\"") + af_version() +"\"",false);
#endif
	pop();
    }

    void add_DFXML_execution_environment(const std::string &command_line){
	push("execution_environment");
#ifdef HAVE_SYS_UTSNAME_H
	struct utsname name;
	if(uname(&name)==0){
	    xmlout("os_sysname",name.sysname);
	    xmlout("os_release",name.release);
	    xmlout("os_version",name.version);
	    xmlout("host",name.nodename);
	    xmlout("arch",name.machine);
	}
#else
#ifdef UNAMES
	xmlout("os_sysname",UNAMES,"",false);
#endif
#endif	

#ifndef WIN32
	xmlout("command_line", command_line); // quote it!
	xmlprintf("uid","","%d",getuid());
	xmlout("username",getpwuid(getuid())->pw_name);
#endif
	pop();			// <execution_environment>
    }
};

#endif
